//
//  main.m
//  diff_audio
//
//  Created by Julien-Pierre Avérous on 11/03/2015.
//  Copyright (c) 2015 Julien-Pierre Avérous. All rights reserved.
//

@import Foundation;
@import AVFoundation;

int main(int argc, const char * argv[])
{
	// Check argument.
	if (argc != 4)
	{
		fprintf(stderr, "Usage: diff_audio file1 file2 diff\n");
		return 1;
	}
	
	@autoreleasepool
	{
		NSError *error = nil;
		
		// Get argument.
		NSString *file1Path = @(argv[1]);
		NSString *file2Path = @(argv[2]);
		NSString *diffPath = @(argv[3]);

		// Open audio file1.
		AVAudioFile *audioFile1 = [[AVAudioFile alloc] initForReading:[NSURL fileURLWithPath:file1Path] commonFormat:AVAudioPCMFormatFloat32 interleaved:NO error:&error];
		
		if (!audioFile1)
		{
			fprintf(stderr, "Error: Can't open '%s' (%s).\n", file1Path.fileSystemRepresentation, error.localizedDescription.UTF8String);
			return 2;
		}
		
		// Open audio file2.
		AVAudioFile *audioFile2 = [[AVAudioFile alloc] initForReading:[NSURL fileURLWithPath:file2Path] commonFormat:AVAudioPCMFormatFloat32 interleaved:NO error:&error];
		
		if (!audioFile2)
		{
			fprintf(stderr, "Error: Can't open '%s' (%s).\n", file2Path.fileSystemRepresentation, error.localizedDescription.UTF8String);
			return 2;
		}

		// Create diff file.
		AVAudioFile *audioDiff = [[AVAudioFile alloc] initForWriting:[NSURL fileURLWithPath:diffPath]
															settings:@{ AVFormatIDKey: @(kAudioFormatAppleLossless),  AVSampleRateKey : @44100, AVNumberOfChannelsKey : @2 }
														commonFormat:AVAudioPCMFormatFloat32
														 interleaved:NO
															   error:&error];
		
		if (!audioDiff)
		{
			fprintf(stderr, "Error: Can't create '%s' (%s).\n", diffPath.fileSystemRepresentation, error.localizedDescription.UTF8String);
			return 2;
		}

		// Create buffers.
		AVAudioFormat *audioFormat = [[AVAudioFormat alloc] initWithCommonFormat:AVAudioPCMFormatFloat32 sampleRate:44100 channels:2 interleaved:NO];

		AVAudioPCMBuffer *file1Buffer = [[AVAudioPCMBuffer alloc] initWithPCMFormat:audioFormat frameCapacity:10000];
		AVAudioPCMBuffer *file2Buffer = [[AVAudioPCMBuffer alloc] initWithPCMFormat:audioFormat frameCapacity:10000];
		AVAudioPCMBuffer *diffBuffer = [[AVAudioPCMBuffer alloc] initWithPCMFormat:audioFormat frameCapacity:10000];


		while (1)
		{
			// Read frames.
			if ([audioFile1 readIntoBuffer:file1Buffer error:nil] == NO || [audioFile2 readIntoBuffer:file2Buffer error:nil] == NO)
			{
				fprintf(stderr, "Error: Can't read input files.\n");
				return 3;
			}
			
			// Get max length.
			AVAudioFrameCount frameLength = MAX(file1Buffer.frameLength, file2Buffer.frameLength);

			if (frameLength == 0)
				break;
			
			diffBuffer.frameLength = frameLength;
			
			// Get float buffers.
			const float *file1Chan1 = file1Buffer.floatChannelData[0];
			const float *file1Chan2 = file1Buffer.floatChannelData[1];
			
			const float *file2Chan1 = file2Buffer.floatChannelData[0];
			const float *file2Chan2 = file2Buffer.floatChannelData[1];

			float *diffChan1 = diffBuffer.floatChannelData[0];
			float *diffChan2 = diffBuffer.floatChannelData[1];
			
			for (AVAudioFrameCount i = 0; i < frameLength; i++)
			{
				// Get value.
				float file1ChanVal1 = 0.0;
				float file1ChanVal2 = 0.0;

				float file2ChanVal1 = 0.0;
				float file2ChanVal2 = 0.0;

				if (i < file1Buffer.frameLength)
				{
					file1ChanVal1 = file1Chan1[i];
					file1ChanVal2 = file1Chan2[i];
				}
				
				if (i < file2Buffer.frameLength)
				{
					file2ChanVal1 = file2Chan1[i];
					file2ChanVal2 = file2Chan2[i];
				}
				
				// Diff.
				float diffChanVal1 = file1ChanVal1 - file2ChanVal1;
				float diffChanVal2 = file1ChanVal2 - file2ChanVal2;
				
				diffChan1[i] = diffChanVal1;
				diffChan2[i] = diffChanVal2;
			}
			
			// Write.
			if ([audioDiff writeFromBuffer:diffBuffer error:nil] == NO)
			{
				fprintf(stderr, "Error: Can't write output file.\n");
				return 4;
			}
		}
		
		fprintf(stderr, "Done.\n");
	}
    return 0;
}
